; ******************************************************************************
; Cottontail Boot Loader Stub Code
; Description:
; Contains procedures that are most easily written in assembly. Procedures are
; called using the 'C' calling convention. AX, CX, DX, are all clobbered.
; ******************************************************************************

[bits 16]

[global _start]
_start:		; first-stage bootloader transfers control to us here; 0000:0600
mov ax, cs	; set up segments
mov ds, ax
mov es, ax
mov ss, ax
mov [_data_seg], ax
xor sp, sp
sti

xor dh, dh
push word oooo
push word ssss	; so we know where to jump at the end
push dx		; push drive number we are booting from
push bx		; push pointer to boot sector received from 1st stage bootloader
[extern _cbl_main]
call _cbl_main	; call C code to start the Cottontail Boot Loader
add sp, 8	; adjust the stack

[extern _mods]
mov bx, _mods	; save a pointer to the 'mods' structure
mov cx, [oooo]
mov [long_jump+1], cx
mov cx, [ssss]
mov [long_jump+3], cx	; set up for the long jump
jmp short long_jump	; clear PFIQ

long_jump:
jmp 0000:0000	; long jump to start the module

oooo: dw 0
ssss: dw 0

[global _data_seg]
_data_seg: dw 0

[global _putc]
_putc:
push bp
mov bp, sp
mov al, [bp+4]	; get character
push bx
mov bx, 0x03
mov ah, 0x0E
int 0x10	; draw character
pop bx
pop bp
ret

[global _bios_reset_disk_system]
_bios_reset_disk_system:
push bp
mov bp, sp
mov dl, [bp+4]	; get drive #
mov ah, 0x00
int 0x13
pop bp
shr ax, 8
ret

[global _bios_get_disk_status]
_bios_get_disk_status:
push bp
mov bp, sp
mov dl, [bp+4]	; get drive #
mov ah, 0x01
int 0x13
pop bp
shr ax, 8
ret

[global _bios_read_disk_old_chs]
_bios_read_disk_old_chs:
push bp
mov bp, sp
mov dl, [bp+4]	; get drive #
mov ch, [bp+6]	; get cylinder #
mov cl, [bp+7]
shl cl, 6
mov dh, [bp+8]	; get head #
or cl, [bp+10]	; get sector #
mov al, [bp+12]	; get number of sectors to read
push es
push ax
mov ax, [bp+14]	; get segment
mov es, ax
pop ax
mov bx, [bp+16]	; get offset
mov ah, 0x02
int 0x13
pop es
pop bp
jnc _B0001
shr ax, 8
jmp short _B0002
_B0001:
xor ax, ax
_B0002:
ret

[global _bios_get_drive_parms_old_chs]
_bios_get_drive_parms_old_chs:
push bp
mov bp, sp
mov dl, [bp+4]	; get drive #
push es
push di
xor ax, ax
mov es, ax
mov di, ax	; zero ES:DI
push bx
mov ah, 0x08
int 0x13
mov bx, [bp+6]	; get pointer to max cylinder
mov [bx], ch
mov [bx+1], cl
shr byte [bx+1], 6
mov bx, [bp+8]	; get pointer to max head
mov [bx], dh
mov bx, [bp+10]	; get pointer to max sector
and cl, 0x3F
mov [bx], cl
pop bx
pop di
pop es
pop bp
shr ax, 8
ret

[global _bios_read_disk_ext]
_bios_read_disk_ext:
push bp
mov bp, sp
mov dl, [bp+4]	; get drive #
push si
mov si, [bp+6]	; get pointer to packet
mov ah, 0x42
int 0x13
pop si
pop bp
shr ax, 8
ret

[global _bios_get_drive_parms_ext]
_bios_get_drive_parms_ext:
push bp
mov bp, sp
mov dl, [bp+4]	; get drive #
push si
mov si, [bp+6]	; get pointer to drive parm buffer
mov ah, 0x48
int 0x13
pop si
pop bp
shr ax, 8
ret

[global _bios_getch]
_bios_getch:
push bp
mov bp, sp
xor ah, ah
int 0x16
and ax, 0xFF
pop bp
ret

[global _lmul]
_lmul:
push bp
mov bp, sp
push bx
xor ebx, ebx
mov bx, [bp+4]
mov eax, [bx]
mov bx, [bp+6]
mov ecx, [bx]
xor edx, edx
mul ecx
mov bx, [bp+4]
mov [bx], eax
pop bx
pop bp
ret

[global _ldiv]
_ldiv:
push bp
mov bp, sp
push bx
xor ebx, ebx
mov bx, [bp+4]
mov eax, [bx]
mov bx, [bp+6]
mov ecx, [bx]
xor edx, edx
div ecx
mov bx, [bp+4]
mov [bx], eax
mov bx, [bp+8]
mov [bx], edx
pop bx
pop bp
ret

[global _llsh]
_llsh:
push bp
mov bp, sp
push bx
mov bx, [bp+4]
mov cl, [bp+6]
shl dword [bx], cl
pop bx
pop bp
ret

[global _lrsh]
_lrsh:
push bp
mov bp, sp
push bx
mov bx, [bp+4]
mov cl, [bp+6]
shr dword [bx], cl
pop bx
pop bp
ret

[global _pokeb]
_pokeb:
push bp
mov bp, sp
mov ax, [bp+4]
mov fs, ax
push bx
mov bx, [bp+6]
mov al, [bp+8]
mov [fs:bx], al
pop bx
pop bp
ret
